import os

from sqlalchemy import create_engine
from sqlalchemy.orm import sessionmaker

from .models.implementation import Base


def setup_database(logger, db_filename):

    # https://docs.sqlalchemy.org/en/latest/dialects/sqlite.html
    url = "sqlite+pysqlite:///" + db_filename

    engine = create_engine(url, echo=False)

    if not os.path.exists(db_filename):
        logger.info("Initializing new database %s", db_filename)
        init_db(engine)

    Session = sessionmaker(bind=engine)
    session = Session()
    return session


def init_db(engine):
    Base.metadata.create_all(engine)




try:
    from setuptools import setup
except ImportError:
    from distutils.core import setup


with open('README.rst') as readme_file:
    readme = readme_file.read()


requirements = [
    'web3[tester]',  # Cannot have more specific web3 settings separately in test_requirements
    'coloredlogs',
    'colorama',
    'tabulate',
    'sqlalchemy>=1.3.0b1',
    'tqdm',
    'configobj',
    'click',
]

test_requirements = [
    'pytest',
]

dev_requirements = [
    "bumpversion>=0.5.3,<1",
    "wheel",
]

setup(
    name='sto',
    version='0.1.0',
    description="Security token management tool",
    long_description=readme + '\n\n',
    author="TokenMarket Ltd.",
    author_email='mikko@tokenmarket.net',
    url='https://tokenmarket.net',
    packages=[
        'sto',
    ],
    package_dir={'sto':
                 'sto'},
    include_package_data=True,
    install_requires=requirements,
    license="Apache 2.0",
    zip_safe=False,
    keywords='ethereum blockchain smartcontract security token',
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'Natural Language :: English',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.6',
    ],
    test_suite='tests',
    setup_requires=["pytest-runner"],
    extras_require={
        "test": test_requirements,
        "dev": dev_requirements,
    },
    entry_points='''
    [console_scripts]
    sto=sto.cli.main:main
    ''',
)





# https://stackoverflow.com/a/1551394/315168
def pretty_date(time=False):
    """
    Get a datetime object or a int() Epoch timestamp and return a
    pretty string like 'an hour ago', 'Yesterday', '3 months ago',
    'just now', etc
    """
    from datetime import datetime
    now = datetime.utcnow()
    if type(time) is int:
        diff = now - datetime.fromtimestamp(time)
    elif isinstance(time,datetime):
        diff = now - time
    elif not time:
        diff = now - now
    second_diff = diff.seconds
    day_diff = diff.days

    if day_diff < 0:
        return ''

    if day_diff == 0:
        if second_diff < 10:
            return "just now"
        if second_diff < 60:
            return str(second_diff) + " seconds ago"
        if second_diff < 120:
            return "a minute ago"
        if second_diff < 3600:
            return str(second_diff / 60) + " minutes ago"
        if second_diff < 7200:
            return "an hour ago"
        if second_diff < 86400:
            return str(second_diff / 3600) + " hours ago"
    if day_diff == 1:
        return "Yesterday"
    if day_diff < 7:
        return str(day_diff) + " days ago"
    if day_diff < 31:
        return str(day_diff / 7) + " weeks ago"
    if day_diff < 365:
        return str(day_diff / 30) + " months ago"
    return str(day_diff / 365) + " years ago"